-module(state).
-export([main/0]).

% Simple server that can receive two types of messages:
%    {hello, From, Name}, where From is the PID of the sender
%    {reqs, From},  where From is the PID of the sender
% If we get {hello, From, Name}:
%    - Already communicated with From and same Name return: "Already talked with you"
%    - Already communicated with From and diff Name return: "No you're not"
%    - First communication with From return: "Hello mate!"
% If we get {reqs, From}, we return the total number of contacts we have until that point.

server(ContactsNumber,ListOfContacts) ->
    receive
        {hello, From, Name} ->
	    search(From,Name,ContactsNumber,ListOfContacts,ListOfContacts);
        {reqs, From} ->
	    From ! ContactsNumber,
	    server(ContactsNumber,ListOfContacts)
    end.

search(From,Name,ContactsNumber,FullListOfContacts,[{From,Name}| _Tail]) ->
    From ! "Already talked with you!",
    server(ContactsNumber,FullListOfContacts);
search(From,_Name,ContactsNumber,FullListOfContacts,[{From,_}| _Tail]) ->
    From ! "No you're not",
    server(ContactsNumber,FullListOfContacts);
search(From,Name,ContactsNumber,FullListOfContacts,[_H | Tail]) ->
    search(From,Name,ContactsNumber,FullListOfContacts,Tail);
search(From,Name,ContactsNumber,FullListOfContacts,[]) ->
    From ! "Nice to meet you!",
    server(ContactsNumber+1,[{From,Name} | FullListOfContacts]).

client(Server,Name,OtherName) ->
   Server ! { hello,self(),Name},
   receive Msg1 -> io:fwrite("~p: ~s~n", [Name,Msg1]) end,
   Server ! { hello,self(),OtherName},
   receive Msg2 -> io:fwrite("~p: ~s~n", [Name, Msg2]) end,
   Server ! { hello, self(),Name},
   receive Msg3 -> io:fwrite("~p: ~s~n", [Name, Msg3]) end.
    

main() ->
    Server = spawn(fun() -> server(0,[]) end),
    Xs = lists:seq(1, 10), % Creates a list from 1 to 10
    [ spawn(fun() -> client(Server,X,X+1) end) || X <- Xs],
    ok.
