-module(server_powers2).
-export([main/0]).

start_server() ->
    Pid = spawn(fun server_fun/0),
    catch unregister(server),
    register(server, Pid).

% The messages to the server now also contains a reference
server_fun() ->
    receive
        {compute, Fun, Args, From, Ref} ->
            spawn(
              fun() ->
                      Ans = apply(Fun, Args),
                      timer:sleep(round(Ans) * 10),
                      From ! {answer, Ans, Ref}
              end)
    end,
    server_fun().


powers(Nums, Exp) ->
    Fun = fun(X, Y) -> round(math:pow(X, Y)) end,
    % We generate a new reference for number in Nums
    RefsNums = [{make_ref(), Num} || Num <- Nums],
    % Send the messages with the reference added to them
    [server ! {compute, Fun, [Num, Exp], self(), Ref} || {Ref, Num} <- RefsNums],
    % Receive messages in the correct order
    Ans = [receive {answer, Ans, Ref} -> Ans end || {Ref, _} <- RefsNums],
    io:fwrite("~p ^ ~p = ~p~n", [Nums, Exp, Ans]).


main() ->
    start_server(),

    powers([1,2,3,4], 3),
    powers([7,0,5,2], 3),
    powers([4,3,2,1], 3).
